import { Client, Collection, GuildMember, TextChannel } from "discord.js";
import SimpleLogger from "./classes/SimpleLogger";
import SimpleCommand from "./classes/SimpleCommand";
import AutoRegister from "./functions/AutoRegister";
import { config } from "./types/ConfigNames";
import SimpleConfig from "./classes/SimpleConfig";

// @Exclude-API
export default class App {

    public client: Client;
    public commands: Collection<string, SimpleCommand> = new Collection();
    public config: Collection<config, SimpleConfig> = new Collection();

    private commandSettings: {registerAllGuilds: boolean; guilds?: string[]};

    constructor(client: Client, token: string, commandSettings: {registerAllGuilds: boolean; guilds?: string[]}) {
        this.client = client;
        this.commandSettings = commandSettings;
        this.start(token);
    }

    private async start(token: string) {
        await this.client.login(token)
        AutoRegister(this).then(() => SimpleLogger.log("AutoRegister Complete"));
        this.onCommand()
    }

    onCommand()  {
        this.client.on("interactionCreate", async interaction => {
            if (!interaction.isCommand()) return;
            const name = interaction.commandName;
            const command = this.commands.get(name);
            if (!command) return;
            if (interaction.guild == null) return;
            await command.init(interaction.member as GuildMember, interaction.guild, interaction.channel as TextChannel, interaction);
            command.execute(this, interaction);
        });
    }

    getCommandSettings() {
        return this.commandSettings;
    }
}