import fs from "fs";
import Common from "../functions/Common";
import { currNames } from "../types/ConfigNames";
import SimpleLogger from "./SimpleLogger";

export default abstract class SimpleConfig {

    public key: string;
    private path: string;
    private config: JSON = JSON.parse("{}");

    constructor(key: string, path: string) {
        this.key = key;
        this.path = path;
        this.init();
        this.updateTypes();
    }

    private async init() {
        if (!fs.existsSync(this.path)) {
            await fs.writeFileSync(this.path, JSON.stringify({}));
        }

        this.loadConfig();
    }

    private async updateTypes() {
        let path = process.cwd() + "/src/types/ConfigNames.ts";
        if (!fs.existsSync(path)) {
            path = process.cwd() + "/node_modules/Foundation/types/ConfigNames.ts";
        }

        if (!fs.existsSync(path)) {
            SimpleLogger.log("Failed to save new config typings!");
        }
        
        let arr = currNames;
        if (arr.includes(this.key)) return;
        arr.push(this.key);

        let str = `const currNames = [${arr.map(r => `"${r}"`).join(", ")}];`;
        str += `\ntype config = ${arr.map(r => `"${r}"`).join("|")};`;
        str += `\nexport {config, currNames};`

        fs.writeFileSync(path, str);
    }

    private async loadConfig() {
        const file = fs.readFileSync(this.path, "utf-8");
        const json = JSON.parse(file);

        if (json == {} || json == null || json == undefined) {
            SimpleLogger.log("Configuration is empty -" + this.path)
            return this.config = JSON.parse("{}");
        }

        SimpleLogger.info("Automatically loaded configuration -" + this.path);
        return this.config = json;
    }

    async update(updatedSchema: JSON, fileName: string) {
        this.config = updatedSchema;
        fs.writeFileSync(this.path, JSON.stringify(updatedSchema, null, 4));
        SimpleLogger.log(`${this.path} -> Configuration updated by ${fileName}.js`);
        return;
    }

    get(path: string) {
        let split = path.split(".");
        if (split.length <= 1) {
            return this.config[split.shift()];
        }
        return this.findPath(split);
    }

    isExists(path: string) {
        return !Common.isNull(this.get(path));
    }

    private findPath(property: string[]) {
        var data = this.config;
        for (const r of property) {
            data = data[r];
        }

        return data;
    }
}