import fs from "fs";
import App from "../App";
import SimpleCommand from "../classes/SimpleCommand";
import SimpleConfig from "../classes/SimpleConfig";
import SimpleEvent from "../classes/SimpleEvent";
import SimpleLogger from "../classes/SimpleLogger";
import { config } from "../types/ConfigNames";

// @Exclude-API
export default async function (app: App) {
    const files = (await fetchFiles(process.cwd())).filter(r => (r.endsWith("js") || r.endsWith("ts")));
    if (files.length == 0) {
        return SimpleLogger.log("AutoRegister found 0 files in main directory.")
    }

    const events: string[] = [];
    const commands: string[] = [];

    for (const filePath of files) {
        if (!fs.existsSync(filePath)) continue;
        const file = fs.readFileSync(filePath, "utf-8");
        if (!file.includes("@AutoRegister") && !file.includes("@Config")) continue;
        if (file.includes("@Exclude-API")) continue;

        const importFile = await (await import(filePath)).default;
        const Class = new importFile();

        if (Class instanceof SimpleEvent) {
            events.push(filePath)
        } else if (Class instanceof SimpleCommand) {
            commands.push(filePath)
        } else if (Class instanceof SimpleConfig) {
            app.config.set(Class.key as config, Class);
        }
    }

    if (events.length !== 0) {
        for (const eventPath of events) {
            if (!fs.existsSync(eventPath)) continue;
            const file = (await import(eventPath))?.default;
            const event = new file();
            app.client.on(event.name, event.execute.bind(null, app))
            SimpleLogger.info("Automatically loaded listener -" + eventPath)
        }
    }

    if (commands.length !== 0) {
        for (const commandPath of commands) {
            if (!fs.existsSync(commandPath)) continue;
            const file = (await import(commandPath))?.default;
            const command = new file(app);
            app.commands.set(command.name, command);
            SimpleLogger.info("Automatically loaded command -" + commandPath)
        }

        const guilds: string[] = app.getCommandSettings().registerAllGuilds ?
            app.client.guilds.cache.toJSON().map(r => r.id) :
            app.getCommandSettings()?.guilds ?? [];

        for (const guildId of guilds) {
            const guild = app.client.guilds.cache.get(guildId);
            guild?.commands.set(app.commands.map(r => {
                return {
                    name: r.name,
                    description: r.description,
                    options: r.options
                }
            }))
        }
    }

    return;
}

async function fetchFiles(path: string) {
    const baseFilter = ["node_modules", "package-lock.json", "package.json"];
    const baseFilesUnformatted = fs.readdirSync(path).filter(r => !baseFilter.includes(r));
    const baseFiles: string[] = [];

    for (const baseFile of baseFilesUnformatted) {
        baseFiles.push(path + "\\" + baseFile);
    }

    let finalFiles: string[] = [];

    for (const baseFile of baseFiles) {
        if (!fs.existsSync(baseFile)) continue;
        if (fs.statSync(baseFile).isDirectory()) {
            finalFiles = finalFiles.concat(await fetchFiles(baseFile));
        } else {
            finalFiles.push(baseFile);
        }
    }

    return finalFiles;


}